import React, { useState, useEffect } from 'react';
import Pagination from './Pagination';

interface TableProps {
  data: any[];
  itemsPerPage: number;
}

const Table: React.FC<TableProps> = ({ data, itemsPerPage }) => {

  const originalData = [...data];

  const [sortedData, setSortedData] = useState<any[]>(data);
  const [sortOrder, setSortOrder] = useState<'asc' | 'desc'>('asc');
  const [sortColumn, setSortColumn] = useState<string | null>(null);
  const [currentPage, setCurrentPage] = useState<number>(1);

  const keys = Object.keys(data[0]);
  const totalPages = Math.ceil(sortedData.length / itemsPerPage);

  useEffect(() => {
    if (sortColumn !== null) {
      const sorted = [...data].sort((a, b) => {
        if (sortOrder === 'asc') {
          return a[sortColumn] < b[sortColumn] ? -1 : 1;
        } else {
          return a[sortColumn] > b[sortColumn] ? -1 : 1;
        }
      });
      setSortedData(sorted);
    }
  }, [sortOrder, data, sortColumn]);

  useEffect(() => {
    setCurrentPage(1); // Reset to the first page when sorting or itemsPerPage changes
  }, [sortColumn, sortOrder, itemsPerPage]);

  const onPageChange = (page: number) => {
    setCurrentPage(page);
  };

  const toggleSort = (column: string) => {
    if (sortColumn === column) {
      if (sortOrder === 'asc') {
        setSortOrder('desc');
        setSortedData([...sortedData].reverse()); // Reverse the sorted data
      } else if (sortOrder === 'desc') {
        setSortColumn(null);
        setSortOrder('asc');
        setSortedData(originalData); // Reset to the original order
      }
    } else {
      setSortColumn(column);
      setSortOrder('asc');
      const sorted = [...originalData].sort((a, b) => {
        return a[column] < b[column] ? -1 : 1;
      });
      setSortedData(sorted);
    }
  };

  // Calculate the startIndex and endIndex based on currentPage and itemsPerPage
  const startIndex = (currentPage - 1) * itemsPerPage;
  const endIndex = startIndex + itemsPerPage;
  const currentData = sortedData.slice(startIndex, endIndex);

  function isString(value: any): boolean {
    return typeof value === 'string';
  }

  return (
    <div className = "text-center font-inter text-primary">
      <table className="text-center my-2 mx-auto table-fixed w-11/12" >
        <thead className="thead">
          <tr className="bg-white text-center p-0 pl-2 border-0 h-2 font-weight: 700">
            {keys.map((item, index) => (
              <th
                className="th"
                key={index}
                onClick={() => toggleSort(item)}
              >
                {item}{' '}
                {sortColumn === item && (
                  <span>{sortOrder === 'asc' ? '▲' : '▼'}</span>
                )
                }
              </th>
            ))}
          </tr>
        </thead>
        <tbody className="border-l-2 border-t-2 border-r-2 border-b-2 border-white w-20">
          {currentData.map((obj, index) => (
            <tr key={index} className={index % 2 === 0 ? "bg-secondary" : "bg-base-100"}>
              {keys.map((item, index) => (
                <td className={`${isString(obj[item]) ? 'text-left' : 'text-center'} px-1 py-0.5 border border-white m-0 overflow-x-hidden truncate`}
                    key={index}
                    >
                    {obj[item]}
                </td>
              ))
              }
            </tr>
          ))}
        </tbody>
      </table>

      <Pagination
        currentPage={currentPage}
        totalPages={totalPages}
        onPageChange={onPageChange}
        itemsPerPageInput = {itemsPerPage}
        numItemsArrayReal = {startIndex+currentData.length}
        totalItems={sortedData.length}
      />
      
      
    </div>
  );
};

export default Table;
